from django.test import TestCase
from django.urls import reverse
from rest_framework.test import APIClient
from rest_framework import status
from phil_loc.models import Barangay, Municipality, Province, Region, PhAddress


class RegionModelTest(TestCase):

    def setUp(self):
        self.region = Region.objects.create(
            name="Test Region",
            psgc_code=123456,
            reg_code=12
        )

    def test_region_creation(self):
        self.assertEqual(self.region.name, "Test Region")
        self.assertEqual(self.region.psgc_code, 123456)
        self.assertEqual(self.region.reg_code, 12)
        self.assertTrue(isinstance(self.region, Region))
        self.assertEqual(str(
            self.region), f"{self.region.pk} | {self.region.reg_code} - {self.region.name}")


class ProvinceModelTest(TestCase):

    def setUp(self):
        self.region = Region.objects.create(
            name="Test Region",
            psgc_code=123456,
            reg_code=12
        )
        self.province = Province.objects.create(
            name="Test Province",
            psgc_code=654321,
            prov_code=21,
            reg_code=12,
            region=self.region
        )

    def test_province_creation(self):
        self.assertEqual(self.province.name, "Test Province")
        self.assertEqual(self.province.psgc_code, 654321)
        self.assertEqual(self.province.prov_code, 21)
        self.assertEqual(self.province.region, self.region)
        self.assertTrue(isinstance(self.province, Province))
        self.assertEqual(str(
            self.province), f"{self.province.pk} | {self.province.prov_code} - {self.province.name}")


class MunicipalityModelTest(TestCase):

    def setUp(self):
        self.region = Region.objects.create(
            name="Test Region",
            psgc_code=123456,
            reg_code=12
        )
        self.province = Province.objects.create(
            name="Test Province",
            psgc_code=654321,
            prov_code=21,
            reg_code=12,
            region=self.region
        )
        self.municipality = Municipality.objects.create(
            name="Test Municipality",
            psgc_code=789012,
            city_mun_code=34,
            prov_code=21,
            reg_code=12,
            province=self.province
        )

    def test_municipality_creation(self):
        self.assertEqual(self.municipality.name, "Test Municipality")
        self.assertEqual(self.municipality.psgc_code, 789012)
        self.assertEqual(self.municipality.city_mun_code, 34)
        self.assertEqual(self.municipality.province, self.province)
        self.assertTrue(isinstance(self.municipality, Municipality))
        self.assertEqual(str(self.municipality),
                         f"{self.municipality.pk} | {self.municipality.city_mun_code} - {self.municipality.name}")


class BarangayModelTest(TestCase):

    def setUp(self):
        self.region = Region.objects.create(
            name="Test Region",
            psgc_code=123456,
            reg_code=12
        )
        self.province = Province.objects.create(
            name="Test Province",
            psgc_code=654321,
            prov_code=21,
            reg_code=12,
            region=self.region
        )
        self.municipality = Municipality.objects.create(
            name="Test Municipality",
            psgc_code=789012,
            city_mun_code=34,
            prov_code=21,
            reg_code=12,
            province=self.province
        )
        self.barangay = Barangay.objects.create(
            name="Test Barangay",
            brgy_code=987654,
            city_mun_code=34,
            prov_code=21,
            reg_code=12,
            municipality=self.municipality
        )

    def test_barangay_creation(self):
        self.assertEqual(self.barangay.name, "Test Barangay")
        self.assertEqual(self.barangay.brgy_code, 987654)
        self.assertEqual(self.barangay.municipality, self.municipality)
        self.assertTrue(isinstance(self.barangay, Barangay))
        self.assertEqual(str(
            self.barangay), f"{self.barangay.pk} | {self.barangay.brgy_code} - {self.barangay.name}")


class PhAddressModelTest(TestCase):

    def setUp(self):
        self.region = Region.objects.create(
            name="Test Region",
            psgc_code=123456,
            reg_code=12
        )
        self.province = Province.objects.create(
            name="Test Province",
            psgc_code=654321,
            prov_code=21,
            reg_code=12,
            region=self.region
        )
        self.municipality = Municipality.objects.create(
            name="Test Municipality",
            psgc_code=789012,
            city_mun_code=34,
            prov_code=21,
            reg_code=12,
            province=self.province
        )
        self.barangay = Barangay.objects.create(
            name="Test Barangay",
            brgy_code=987654,
            city_mun_code=34,
            prov_code=21,
            reg_code=12,
            municipality=self.municipality
        )
        self.ph_address = PhAddress.objects.create(
            region=self.region,
            province=self.province,
            municipality=self.municipality,
            barangay_district=self.barangay,
            unit_home_street="123 Main St",
            zip_code=1000,
        )

    def test_ph_address_creation(self):
        self.assertEqual(self.ph_address.region, self.region)
        self.assertEqual(self.ph_address.province, self.province)
        self.assertEqual(self.ph_address.municipality, self.municipality)
        self.assertEqual(self.ph_address.barangay_district, self.barangay)
        self.assertEqual(self.ph_address.unit_home_street, "123 Main St")
        self.assertEqual(self.ph_address.zip_code, 1000)
        self.assertTrue(isinstance(self.ph_address, PhAddress))
        self.assertEqual(str(self.ph_address), f"{self.ph_address.pk}")


class LocationAPITestCase(TestCase):
    def setUp(self):
        self.client = APIClient()

        # Create test data
        self.region = Region.objects.create(name="Test Region", reg_code=1)
        self.province = Province.objects.create(
            name="Test Province",
            reg_code=self.region.reg_code,
            prov_code=1,
            region=self.region
        )
        self.municipality = Municipality.objects.create(
            name="Test Municipality",
            city_mun_code=1,
            prov_code=self.province.prov_code,
            province=self.province
        )
        self.barangay = Barangay.objects.create(
            name="Test Barangay",
            brgy_code=1,
            city_mun_code=self.municipality.city_mun_code,
            municipality=self.municipality
        )

    def test_region_list(self):
        url = reverse('locations:region-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertContains(response, self.region.name)

    def test_province_list(self):
        url = reverse('locations:province-list',
                      kwargs={'reg_code': self.region.reg_code})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertContains(response, self.province.name)

    def test_municipality_list(self):
        url = reverse('locations:municipality-list',
                      kwargs={'prov_code': self.province.prov_code})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertContains(response, self.municipality.name)

    def test_barangay_list(self):
        url = reverse('locations:barangays-list',
                      kwargs={'city_mun_code': self.municipality.city_mun_code})
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertContains(response, self.barangay.name)



from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class Barangay(models.Model):
    class Meta:
        db_table = 'db_barangay'
        ordering = ['-pk']
        verbose_name = "Barangay"
        verbose_name_plural = "Barangays"

    name = models.CharField(max_length=64)
    brgy_code = models.IntegerField(blank=True, null=True)
    city_mun_code = models.IntegerField(blank=True, null=True)
    prov_code = models.IntegerField(blank=True, null=True)
    reg_code = models.IntegerField(blank=True, null=True)
    municipality = models.ForeignKey(
        "Municipality",
        verbose_name=_("Municipality"),
        db_column='municipality_id',
        related_name='barangay_municipality',
        on_delete=models.CASCADE
    )

    def __str__(self) -> str:
        return f"{self.pk} | {self.brgy_code} - {self.name}"


class Municipality(models.Model):
    class Meta:
        db_table = 'db_municipality'
        ordering = ['-pk']
        verbose_name = "Municipality"
        verbose_name_plural = "Municipalities"

    name = models.CharField(max_length=64)
    psgc_code = models.IntegerField(blank=True, null=True)
    city_mun_code = models.IntegerField(blank=True, null=True)
    prov_code = models.IntegerField(blank=True, null=True)
    reg_code = models.IntegerField(blank=True, null=True)
    province = models.ForeignKey(
        "Province",
        verbose_name=_("Province"),
        db_column='province_id',
        related_name='municipality_province',
        on_delete=models.CASCADE
    )

    def __str__(self) -> str:
        return f"{self.pk} | {self.city_mun_code} - {self.name}"


class Province(models.Model):
    class Meta:
        db_table = 'db_province'
        ordering = ['-pk']
        verbose_name = "Province"
        verbose_name_plural = "Provinces"

    name = models.CharField(max_length=64)
    psgc_code = models.IntegerField(blank=True, null=True)
    prov_code = models.IntegerField(blank=True, null=True)
    reg_code = models.IntegerField(blank=True, null=True)
    region = models.ForeignKey(
        "Region",
        verbose_name=_("Region"),
        db_column='region_id',
        related_name='province_region',
        on_delete=models.CASCADE
    )

    def __str__(self) -> str:
        return f"{self.pk} | {self.prov_code} - {self.name}"


class COUNTRY(models.TextChoices):
    PH = 'PH', 'Philippines'


class Region(models.Model):
    class Meta:
        db_table = 'db_region'
        ordering = ['-pk']
        verbose_name = "Region"
        verbose_name_plural = "Regions"

    name = models.CharField(max_length=64)
    psgc_code = models.IntegerField(blank=True, null=True)
    reg_code = models.IntegerField(unique=True)
    country = models.CharField(
        "Country",
        max_length=4,
        choices=COUNTRY.choices,
        default=COUNTRY.PH,
    )

    def __str__(self) -> str:
        return f"{self.pk} | {self.reg_code} - {self.name}"


class PhAddress(models.Model):
    class Meta:
        db_table = 'db_ph_address'
        ordering = ['-pk']
        verbose_name = "Phillipines Address"
        verbose_name_plural = "Phillipines Addresses"

    country = models.CharField(
        "Country",
        max_length=4,
        choices=COUNTRY.choices,
        default=COUNTRY.PH,
    )
    region = models.ForeignKey(
        to=Region,
        verbose_name=_("Region"),
        db_column='region_id',
        related_name='address_region',
        on_delete=models.DO_NOTHING,
    )
    province = models.ForeignKey(
        to=Province,
        verbose_name=_("Province"),
        db_column='province_id',
        related_name='address_province',
        on_delete=models.DO_NOTHING,
    )
    municipality = models.ForeignKey(
        to=Municipality,
        verbose_name=_("Municipality"),
        db_column='municipality_id',
        related_name='ph_addresses',
        on_delete=models.DO_NOTHING,
    )
    barangay_district = models.ForeignKey(
        to=Barangay,
        verbose_name=_("Barangay"),
        db_column='barangay_id',
        related_name='address_barangay',
        on_delete=models.DO_NOTHING,
        null=True,
        blank=True
    )
    unit_home_street = models.CharField(
        max_length=256, null=True, default=None)
    zip_code = models.IntegerField(blank=True, null=True, default=None)
    district_id = models.IntegerField(blank=True, null=True, default=None)
    created_at = models.DateTimeField(
        auto_now_add=False, auto_now=False, default=timezone.now)
    updated_at = models.DateTimeField(auto_now_add=True)

    def __str__(self) -> str:
        return f"{self.pk}"



from django.urls import path

from .views import (BarangayListAPIView, CityMunListAPIView,
                               ProvinceListAPIView, RegionListAPIView)

app_name = 'locations'

urlpatterns = [
    path('regions/',
         RegionListAPIView.as_view(), name='region-list'),
    path('regions/<int:reg_code>/provinces/',
         ProvinceListAPIView.as_view(), name='province-list'),
    path('provinces/<int:prov_code>/municipalities/',
         CityMunListAPIView.as_view(), name='municipality-list'),
    path('municipalities/<int:city_mun_code>/barangays/',
         BarangayListAPIView.as_view(), name='barangays-list'),
]



from rest_framework import serializers

from .models import (PhAddress, Barangay, Municipality, Province, Region)


class RegionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Region
        fields = ["id", "name", "reg_code"]


class ProvinceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Province
        fields = ["id", "name", "prov_code"]


class CityMunSerializer(serializers.ModelSerializer):
    class Meta:
        model = Municipality
        fields = ["id", "name", "city_mun_code"]


class BarangaySerializer(serializers.ModelSerializer):
    class Meta:
        model = Barangay
        fields = ["id", "name", "brgy_code"]


class RegionSerializerRepresentation(serializers.ModelSerializer):
    class Meta:
        model = Region
        fields = ["id", "name"]

    def to_representation(self, instance):
        return instance.name


class ProvinceSerializerRepresentation(serializers.ModelSerializer):
    class Meta:
        model = Province
        fields = ["id", "name"]

    def to_representation(self, instance):
        return instance.name


class CityMunSerializerRepresentation(serializers.ModelSerializer):
    class Meta:
        model = Municipality
        fields = ["id", "name"]

    def to_representation(self, instance):
        return instance.name


class BarangaySerializerRepresentation(serializers.ModelSerializer):
    class Meta:
        model = Barangay
        fields = ["id", "name"]

    def to_representation(self, instance):
        return instance.name


class PhAddressUpsertSerializer(serializers.ModelSerializer):
    region_id = serializers.PrimaryKeyRelatedField(
        queryset=Region.objects.all(),
        source='region',
        required=True
    )
    province_id = serializers.PrimaryKeyRelatedField(
        queryset=Province.objects.all(),
        source='province',
        required=True
    )
    municipality_id = serializers.PrimaryKeyRelatedField(
        queryset=Municipality.objects.all(),
        source='municipality',
        required=True
    )
    barangay_district_id = serializers.PrimaryKeyRelatedField(
        queryset=Barangay.objects.all(),
        source='barangay_district',
        required=True
    )

    class Meta:
        model = PhAddress
        fields = [
            'id',
            'country',
            'region_id',
            'province_id',
            'municipality_id',
            'barangay_district_id',
            'unit_home_street',
            'zip_code',
            'district_id',
            'updated_at'
        ]


class PhAddressSerializer(serializers.ModelSerializer):
    region = RegionSerializer()
    province = ProvinceSerializer()
    municipality = CityMunSerializer()
    barangay_district = BarangaySerializer()

    class Meta:
        model = PhAddress
        fields = [
            'id',
            'country',
            'region',
            'province',
            'municipality',
            'barangay_district',
            'unit_home_street',
            'zip_code',
            'district_id',
            'updated_at',
        ]



# Generated by Django 5.1 on 2024-08-29 13:47

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Municipality',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('psgc_code', models.IntegerField(blank=True, null=True)),
                ('city_mun_code', models.IntegerField(blank=True, null=True)),
                ('prov_code', models.IntegerField(blank=True, null=True)),
                ('reg_code', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Municipality',
                'verbose_name_plural': 'Municipalities',
                'db_table': 'db_municipality',
                'ordering': ['-pk'],
            },
        ),
        migrations.CreateModel(
            name='Province',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('psgc_code', models.IntegerField(blank=True, null=True)),
                ('prov_code', models.IntegerField(blank=True, null=True)),
                ('reg_code', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Province',
                'verbose_name_plural': 'Provinces',
                'db_table': 'db_province',
                'ordering': ['-pk'],
            },
        ),
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('psgc_code', models.IntegerField(blank=True, null=True)),
                ('reg_code', models.IntegerField(unique=True)),
                ('country', models.CharField(choices=[('PH', 'Philippines')], default='PH', max_length=4, verbose_name='Country')),
            ],
            options={
                'verbose_name': 'Region',
                'verbose_name_plural': 'Regions',
                'db_table': 'db_region',
                'ordering': ['-pk'],
            },
        ),
        migrations.CreateModel(
            name='Barangay',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
                ('brgy_code', models.IntegerField(blank=True, null=True)),
                ('city_mun_code', models.IntegerField(blank=True, null=True)),
                ('prov_code', models.IntegerField(blank=True, null=True)),
                ('reg_code', models.IntegerField(blank=True, null=True)),
                ('municipality', models.ForeignKey(db_column='municipality_id', on_delete=django.db.models.deletion.CASCADE, related_name='barangay_municipality', to='phil_loc.municipality', verbose_name='Municipality')),
            ],
            options={
                'verbose_name': 'Barangay',
                'verbose_name_plural': 'Barangays',
                'db_table': 'db_barangay',
                'ordering': ['-pk'],
            },
        ),
        migrations.AddField(
            model_name='municipality',
            name='province',
            field=models.ForeignKey(db_column='province_id', on_delete=django.db.models.deletion.CASCADE, related_name='municipality_province', to='phil_loc.province', verbose_name='Province'),
        ),
        migrations.AddField(
            model_name='province',
            name='region',
            field=models.ForeignKey(db_column='region_id', on_delete=django.db.models.deletion.CASCADE, related_name='province_region', to='phil_loc.region', verbose_name='Region'),
        ),
        migrations.CreateModel(
            name='PhAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('country', models.CharField(choices=[('PH', 'Philippines')], default='PH', max_length=4, verbose_name='Country')),
                ('unit_home_street', models.CharField(default=None, max_length=256, null=True)),
                ('zip_code', models.IntegerField(blank=True, default=None, null=True)),
                ('district_id', models.IntegerField(blank=True, default=None, null=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(auto_now_add=True)),
                ('barangay_district', models.ForeignKey(blank=True, db_column='barangay_id', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='address_barangay', to='phil_loc.barangay', verbose_name='Barangay')),
                ('municipality', models.ForeignKey(db_column='municipality_id', on_delete=django.db.models.deletion.DO_NOTHING, related_name='ph_addresses', to='phil_loc.municipality', verbose_name='Municipality')),
                ('province', models.ForeignKey(db_column='province_id', on_delete=django.db.models.deletion.DO_NOTHING, related_name='address_province', to='phil_loc.province', verbose_name='Province')),
                ('region', models.ForeignKey(db_column='region_id', on_delete=django.db.models.deletion.DO_NOTHING, related_name='address_region', to='phil_loc.region', verbose_name='Region')),
            ],
            options={
                'verbose_name': 'Phillipines Address',
                'verbose_name_plural': 'Phillipines Addresses',
                'db_table': 'db_ph_address',
                'ordering': ['-pk'],
            },
        ),
    ]



from rest_framework.generics import ListAPIView

from .models import Barangay, Municipality, Province, Region
from .serializers import (BarangaySerializer, CityMunSerializer,
                           ProvinceSerializer, RegionSerializer)


class RegionListAPIView(ListAPIView):
    serializer_class = RegionSerializer
    queryset = Region.objects.all()


class ProvinceListAPIView(ListAPIView):
    serializer_class = ProvinceSerializer

    def get_queryset(self):
        reg_code = self.kwargs.get('reg_code')
        if reg_code is not None:
            return Province.objects.filter(reg_code=reg_code)
        else:
            return Province.objects.all()


class CityMunListAPIView(ListAPIView):
    serializer_class = CityMunSerializer

    def get_queryset(self):
        prov_code = self.kwargs.get('prov_code')
        if prov_code is not None:
            return Municipality.objects.filter(prov_code=prov_code)
        else:
            return Municipality.objects.all()


class BarangayListAPIView(ListAPIView):
    serializer_class = BarangaySerializer

    def get_queryset(self):
        city_mun_code = self.kwargs.get('city_mun_code')
        if city_mun_code is not None:
            return Barangay.objects.filter(city_mun_code=city_mun_code)
        else:
            return Barangay.objects.all()



import json
import logging

from alive_progress import alive_bar
from django.core.management.base import BaseCommand
from django.db import IntegrityError
from importlib import resources

from ...models import Barangay, Municipality, Province, Region

# python manage.py seed --mode=refresh

""" Clear all data and creates addresses """
MODE_REFRESH = 'refresh'

""" Clear all data and do not create any object """
MODE_CLEAR = 'clear'

data_src = 'phil_loc.static.data'


class Command(BaseCommand):
    help = "seed database for testing and development."

    def add_arguments(self, parser):
        parser.add_argument('--mode', type=str, help="Mode")

    def handle(self, *args, **options):
        self.stdout.write('seeding data...')
        run_seed(self, options['mode'])
        self.stdout.write('done.')


def clear_data():
    """Deletes all the table data"""

    logging.info("Delete Location instances".center(40, '-'))
    Barangay.objects.all().delete()
    Municipality.objects.all().delete()
    Province.objects.all().delete()
    Region.objects.all().delete()


def generate_regions():
    print("GENERATING REGIONS...")
    with resources.open_text(data_src, 'refregion.json') as refregion_file:
        dictionary_list = json.load(refregion_file)['RECORDS']

        total_regions = len(dictionary_list)
        print(f"total_regions :: {total_regions}")

        with alive_bar(total_regions, title="Generating PH REGIONS", title_length=50) as bar:
            for item in dictionary_list:
                id = item["id"]
                name = item["regDesc"]
                regCode = item["regCode"]
                bar.title("Creating Object for {}..".format(name))
                try:
                    if (Region.objects.filter(id=id).count() >= 1):
                        print(f"{name} already exist..")
                    else:
                        Region.objects.create(
                            id=item['id'],
                            name=item['regDesc'],
                            psgc_code=item['psgcCode'],
                            reg_code=regCode,
                            country='PH',
                        )

                except IntegrityError as e:
                    if str(e.__cause__) == "UNIQUE constraint failed: Region.regCode":
                        print("Object: {}..Skipped".format(name))
                    else:
                        print("Object: {}..{}".format(
                            name), e.__cause__)

                bar()


def generate_provinces():
    print("GENERATING Provinces...")
    with resources.open_text(data_src, 'refprovince.json') as refprovince_file:
        dictionary_list = json.load(refprovince_file)['RECORDS']

        total_provinces = len(dictionary_list)
        print(f"total_provinces :: {total_provinces}")

        with alive_bar(total_provinces, title="Generating PH Provinces", title_length=50) as bar:
            for item in dictionary_list:
                id = item["id"]
                name = item["provDesc"]
                reg_code = item["regCode"]
                prov_code = item["provCode"]
                bar.title("Creating Object for {}..".format(name))
                try:
                    if (Province.objects.filter(id=id).count() >= 1):
                        print(f"{name} already exist..")
                    else:
                        region = Region.objects.get(reg_code=reg_code)
                        Province.objects.create(
                            id=id,
                            name=name,
                            psgc_code=item['psgcCode'],
                            prov_code=prov_code,
                            reg_code=reg_code,
                            region=region
                        )

                except IntegrityError as e:
                    if str(e.__cause__) == "UNIQUE constraint failed: Province.prov_code":
                        print("Object: {}..Skipped".format(name))
                    else:
                        print("Object: {}..{}".format(
                            name), e.__cause__)

                bar()


def generate_municipality_cities():
    print("GENERATING municipality_city...")
    with resources.open_text(data_src, 'refcitymun.json') as refmunicipality_city_file:
        dictionary_list = json.load(refmunicipality_city_file)['RECORDS']

        total_municipality_city = len(dictionary_list)
        print(f"total_municipality_citys :: {total_municipality_city}")

        with alive_bar(total_municipality_city, title="Generating PH municipality_citys", title_length=50) as bar:
            for item in dictionary_list:

                id = item["id"]
                name = item["citymunDesc"]
                city_mun_code = item["citymunCode"]
                prov_code = item["provCode"]
                reg_code = item["regCode"]
                bar.title("Creating Object for {}..".format(name))
                try:
                    if (Municipality.objects.filter(id=id).count() >= 1):
                        print(f"{name} already exist..")
                    else:
                        province = Province.objects.filter(
                            prov_code=prov_code).first()
                        Municipality.objects.create(
                            id=id,
                            name=name,
                            psgc_code=item['psgcCode'],
                            city_mun_code=city_mun_code,
                            prov_code=prov_code,
                            reg_code=reg_code,
                            province=province
                        )

                except IntegrityError as e:
                    if str(e.__cause__) == "UNIQUE constraint failed: Municipality.city_mun_code":
                        print("Object: {}..Skipped".format(name))
                    else:
                        print("Object: {}..{}".format(
                            name), e.__cause__)

                bar()


def generate_barangays():
    print("GENERATING barangay...")
    with resources.open_text(data_src, 'refbrgy.json') as refbarangay_file:
        dictionary_list = json.load(refbarangay_file)['RECORDS']

        total_barangay = len(dictionary_list)
        print(f"total_barangays :: {total_barangay}")

        with alive_bar(total_barangay, title="Generating PH barangays", title_length=50) as bar:
            for item in dictionary_list:

                id = item["id"]
                name = item["brgyDesc"]
                brgy_code = item["brgyCode"]
                city_mun_code = item["citymunCode"]
                prov_code = item["provCode"]
                reg_code = item["regCode"]
                bar.title("Creating Object for {}..".format(name))
                try:
                    if (Barangay.objects.filter(id=id).count() >= 1):
                        print(f"{name} already exist..")
                    else:
                        municipality = Municipality.objects.filter(
                            city_mun_code=city_mun_code).first()
                        Barangay.objects.create(
                            id=id,
                            name=name,
                            brgy_code=brgy_code,
                            city_mun_code=city_mun_code,
                            prov_code=prov_code,
                            reg_code=reg_code,
                            municipality=municipality
                        )

                except IntegrityError as e:
                    if str(e.__cause__) == "UNIQUE constraint failed: Barangay.brgy_code":
                        print("Object: {}..Skipped".format(name))
                    else:
                        print("Object: {}..{}".format(
                            name), e.__cause__)

                bar()


def run_seed(self, mode):
    """ Seed database based on mode

    :param mode: refresh / clear 
    :return:
    """
    if mode == MODE_REFRESH:
        clear_data()

    if mode == MODE_CLEAR:
        clear_data()
        return
    generate_regions()
    generate_provinces()
    generate_municipality_cities()
    generate_barangays()
